const buyableItems = require('./buyableItems');
const { commas, saveGame, setHighestCoins, addAchievement, getCoins, getUser, singleItemCps } = require('./utils');
const slack = require('../../slack')

const calculateCost = ({ itemName, user, quantity = 1 }) => {
  let currentlyOwned = user.items[itemName] || 0
  let realCost = 0
  for (let i = 0; i < quantity; i++) {
    realCost += Math.ceil(buyableItems[itemName].baseCost * Math.pow(1.15, currentlyOwned || 0))
    currentlyOwned += 1
  }
  return realCost
}

const getItemHeader = user => ([itemName, { baseCost, description, emoji }]) => {
  const itemCost = commas(user ? calculateCost({ itemName, user }) : baseCost)
  const itemCps = Math.round(singleItemCps(user, itemName))
  return `*${itemName}* :${emoji}: - ${itemCost} HVAC Coins - ${commas(itemCps)} CPS\n_${description}_`
}
const canView = highestCoins => ([, item]) => item.baseCost < (highestCoins || 1) * 101
const buyableText = (highestCoins, user) => Object.entries(buyableItems)
  .filter(canView(highestCoins))
  .map(getItemHeader(user))
  .join('\n\n') +
  '\n\n:grey_question::grey_question::grey_question:' +
  '\n\nJust type \'!buy item_name\' to purchase'

const buildBlock = ({ user, itemName, cost, cps }) => ({
  type: 'section',
  text: {
    type: 'mrkdwn',
    text: `${itemName} :${buyableItems[itemName].emoji}:x${user.items[itemName] || 0} - 𝕳${commas(cost)} - ${commas(cps)} CPS\n_${buyableItems[itemName].description}_`
  },
  accessory: {
    type: 'button',
    text: {
      type: 'plain_text',
      text: 'Buy 1',
      emoji: true
    },
    value: 'buy_' + itemName,
    action_id: 'buy_' + itemName
  },
})

const buyText2 = (highestCoins, user) => {
  return ({
    text: buyableText(highestCoins, user),
    blocks: Object.entries(buyableItems)
    .filter(canView(highestCoins))
    .map(([itemName, item]) => {
      const cost = calculateCost({ itemName, user, quantity: 1 })
      const cps = Math.round(singleItemCps(user, itemName))
      return ({ user, itemName, cost, cps })
    }).map(buildBlock)
  })
}

const buyRoute = async ({ event, say, words }) => {
  const user = getUser(event.user)
  const buying = words[1]
  setHighestCoins(event.user)
  if (!buying) {
    const highestCoins = user.highestEver || user.coins || 1
    if (buyableItems.quade.baseCost < highestCoins * 100) {
      addAchievement(user, 'seeTheQuade', say)
    }
    await say(buyText2(highestCoins, user))
    return
  }

  const buyable = buyableItems[buying]
  if (!buyable) {
    await say('That item does not exist!')
    return
  }

  let quantity = 1
  const currentCoins = getCoins(event.user)
  if (words[2] === 'max') {
    while (calculateCost({ itemName: buying, user, quantity: quantity + 1 }) <= currentCoins) {
      quantity++
    }
  } else {
    quantity = parseInt(words[2] || '1')
  }
  if (!quantity || quantity < 1) {
    await say('Quantity must be a positive integer')
    return
  }

  const realCost = calculateCost({ itemName: buying, user, quantity })
  if (currentCoins < realCost) {
    await say(`You don't have enough coins! You have ${commas(currentCoins)}, but you need ${commas(realCost)}`)
    return
  }
  user.coins -= realCost
  user.items[buying] = user.items[buying] || 0
  user.items[buying] += quantity
  console.log(buying, user.items.mouse)
  if (buying === 'mouse' && user.items.mouse >= 100) {
    addAchievement(user, 'ratGod', say)
  }
  if (quantity === 1) {
    await say(`You bought one :${buyable.emoji}:`)
  } else {
    await say(`You bought ${quantity} :${buyable.emoji}:`)
  }
  saveGame()
}

const buyButton = async ({ body, ack, say, payload }) => {
  await ack()
  const buying = payload.action_id.substring(4)
  console.log(`buyButton ${buying} clicked`)
  const event = {
    user: body.user.id,
  }
  const user = getUser(event.user)
  const words = ['', buying, '1']
  await buyRoute({ event, say, words })
  const highestCoins = user.highestEver || user.coins || 1
  await slack.app.client.chat.update({
    channel: body.channel.id,
    ts: body.message.ts,
    ...buyText2(highestCoins, user)
  })
}

Object.keys(buyableItems).forEach(itemName => slack.app.action('buy_' + itemName, buyButton))

module.exports = buyRoute